@@ -1,8 +1,13 @@
 import time
 import errno
-import socket
 import traceback
+from socket import AF_INET
+from socket import AF_INET6
+from socket import inet_ntop
+from socket import inet_pton
+from socket import error as socket_error
 from pyroute2 import config
+from pyroute2.config import AF_BRIDGE
 from pyroute2.common import basestring
 from pyroute2.common import dqn2int
 from pyroute2.common import View
@@ -103,6 +108,7 @@ class Interface(Transactional):
     _fields.append('kind')
     _fields.append('peer')
     _fields.append('vlan_id')
+    _fields.append('vlan_protocol')
     _fields.append('bond_mode')
     _fields.extend(_get_data_fields())
     _fields.extend(_virtual_fields)
@@ -210,6 +216,8 @@ class Interface(Transactional):
             i.commit()
         '''
         for key in data:
+            if data[key] is None:
+                continue
             if key == 'ipaddr':
                 for addr in self['ipaddr']:
                     self.del_ip(*addr)
@@ -226,31 +234,15 @@ class Interface(Transactional):
                 for vlan in self['vlans']:
                     self.del_vlan(vlan)
                 for vlan in data[key]:
-                    self.add_vlan(vlan)
-            elif key == 'neighbours':
-                # ignore neighbours on load
+                    if vlan != 1:
+                        self.add_vlan(vlan)
+            elif key in ('neighbours', 'family'):
+                # ignore on load
                 pass
             else:
                 self[key] = data[key]
         return self
 
-    def make_transaction(self, data):
-        '''
-        Create a new transaction instance from a dictionary.
-        One can apply it the with `commit(transaction=...)`
-        call.
-
-        Sample::
-
-            data = json.loads(...)
-            t = ipdb.interfaces['dummy1'].make_transaction(data)
-            ipdb.interfaces['dummy1'].commit(transaction=t)
-        '''
-        with self._write_lock:
-            template = self.__class__(ipdb=self.ipdb, mode='snapshot')
-            template.load_dict(data)
-            return template
-
     def load_dict(self, data):
         '''
         Update the interface info from a dictionary.
@@ -305,6 +297,8 @@ class Interface(Transactional):
                     if kind == 'vlan':
                         data = linkinfo.get_attr('IFLA_INFO_DATA')
                         self['vlan_id'] = data.get_attr('IFLA_VLAN_ID')
+                        self['vlan_protocol'] = data\
+                            .get_attr('IFLA_VLAN_PROTOCOL')
                         self['vlan_flags'] = data\
                             .get_attr('IFLA_VLAN_FLAGS', {})\
                             .get('flags', 0)
@@ -316,7 +310,7 @@ class Interface(Transactional):
                             norm = ifinfmsg.nla2name(nla[0])
                             self[norm] = nla[1]
             # load vlans
-            if dev['family'] == socket.AF_BRIDGE:
+            if dev['family'] == AF_BRIDGE:
                 spec = dev.get_attr('IFLA_AF_SPEC')
                 if spec is not None:
                     vlans = spec.get_attrs('IFLA_BRIDGE_VLAN_INFO')
@@ -376,6 +370,10 @@ class Interface(Transactional):
         elif isinstance(mask, basestring):
             mask = dqn2int(mask)
 
+        # normalize the address
+        if ip.find(':') > -1:
+            ip = inet_ntop(AF_INET6, inet_pton(AF_INET6, ip))
+
         # if it is a transaction or an interface update, apply the change
         self['ipaddr'].unlink((ip, mask))
         request = {}
@@ -398,6 +396,9 @@ class Interface(Transactional):
                 mask = dqn2int(mask)
             else:
                 mask = int(mask, 0)
+        # normalize the address
+        if ip.find(':') > -1:
+            ip = inet_ntop(AF_INET6, inet_pton(AF_INET6, ip))
         if (ip, mask) in self['ipaddr']:
             self['ipaddr'].unlink((ip, mask))
             self['ipaddr'].remove((ip, mask))
@@ -512,23 +513,6 @@ class Interface(Transactional):
         if not commit_phase & commit_mask:
             return self
 
-        def invalidate():
-            # on failure, invalidate the interface and detach it
-            # from the parent
-            # 0. obtain lock on IPDB, to avoid deadlocks
-            # ... all the DB updates will wait
-            with self.ipdb.exclusive:
-                # 1. drop the IPRoute() link
-                self.nl = None
-                # 2. clean up ipdb
-                self.detach()
-                # 3. invalidate the interface
-                with self._direct_state:
-                    for i in tuple(self.keys()):
-                        del self[i]
-                # 4. the rest
-                self._mode = 'invalid'
-
         error = None
         added = None
         removed = None
@@ -641,7 +625,7 @@ class Interface(Transactional):
             # Here we come only if a new interface is created
             #
             if commit_phase == 1 and not self.wait_target('ipdb_scope'):
-                invalidate()
+                self.invalidate()
                 if isinstance(newif, Exception):
                     raise newif
                 else:
@@ -667,7 +651,9 @@ class Interface(Transactional):
         if commit_phase == 1 and hasattr(self.ipdb, 'routes'):
             self.routes = []
             for record in self.ipdb.routes.filter({'oif': self['index']}):
-                if record['key'].table != 255:
+                # For MPLS routes the key is an integer
+                # They should match anyways
+                if getattr(record['key'], 'table', None) != 255:
                     self.routes.append((record['route'],
                                         record['route'].pick()))
 
@@ -813,7 +799,7 @@ class Interface(Transactional):
                     any([brequest[item] is not None for item in brequest]):
                 brequest['index'] = self['index']
                 brequest['kind'] = self['kind']
-                brequest['family'] = socket.AF_BRIDGE
+                brequest['family'] = AF_BRIDGE
                 wait_all = True
                 run(nl.link, (RTM_NEWLINK, NLM_F_REQUEST | NLM_F_ACK),
                     **brequest)
@@ -884,7 +870,7 @@ class Interface(Transactional):
                         # 'Cannot assign address'
                         if x.code != errno.EADDRNOTAVAIL:
                             raise
-                    except socket.error as x:
+                    except socket_error as x:
                         # bypass illegal IP requests
                         if isinstance(x.args[0], basestring) and \
                                 x.args[0].startswith('illegal IP'):
@@ -930,7 +916,7 @@ class Interface(Transactional):
                         self['ipaddr'].remove(addr)
                     # 2. reload addresses
                     for addr in self.nl.get_addr(index=self['index'],
-                                                 family=socket.AF_INET6):
+                                                 family=AF_INET6):
                         self.ipdb.ipaddr._new(addr)
                     # if there are tons of IPv6 addresses, it may take a
                     # really long time, and that's bad, but it's broken in
@@ -1158,6 +1144,7 @@ class InterfacesDict(Dotkeys):
                     if self[ifname]['ipdb_scope'] == 'shadow':
                         with device._direct_state:
                             device['ipdb_scope'] = 'create'
+                    device.begin()
                 else:
                     raise CreateException("interface %s exists" %
                                           ifname)
@@ -1173,13 +1160,25 @@ class InterfacesDict(Dotkeys):
                                     kwarg[key].get('ifname')
                         if not isinstance(kwarg[key], int):
                             device._deferred_link = (key, kwarg[key])
-                device.update(kwarg)
-                device['kind'] = kind
-                device['index'] = kwarg.get('index', 0)
-                device['ifname'] = ifname
-                device['ipdb_scope'] = 'create'
                 device._mode = self.ipdb.mode
-            device.begin()
+                with device._direct_state:
+                    device['kind'] = kind
+                    device['index'] = kwarg.get('index', 0)
+                    device['ifname'] = ifname
+                    device['ipdb_scope'] = 'create'
+                    # set some specific attrs
+                    for attr in ('peer',
+                                 'uid',
+                                 'gid',
+                                 'ifr',
+                                 'mode',
+                                 'bond_mode',
+                                 'vrf_table',
+                                 'address'):
+                        if attr in kwarg:
+                            device[attr] = kwarg.pop(attr)
+                device.begin()
+                device.load(kwarg)
         return device
 
     def _del(self, msg):
@@ -1188,7 +1187,7 @@ class InterfacesDict(Dotkeys):
         if target is None:
             return
 
-        if msg['family'] == socket.AF_BRIDGE:
+        if msg['family'] == AF_BRIDGE:
             with target._direct_state:
                 for vlan in tuple(target['vlans']):
                     target.del_vlan(vlan)
@@ -1241,7 +1240,7 @@ class InterfacesDict(Dotkeys):
             if old_index in self.ipdb.neighbours:
                 self.ipdb.neighbours[index] = \
                     self.ipdb.neighbours[old_index]
-                del self.neighbours[old_index]
+                del self.ipdb.neighbours[old_index]
         else:
             # scenario #3, interface rename
             # scenario #4, assume rename
@@ -1330,9 +1329,9 @@ class AddressesDict(dict):
                 iff['ipaddr'] = self[idx]
 
     def _new(self, msg):
-        if msg['family'] == socket.AF_INET:
+        if msg['family'] == AF_INET:
             addr = msg.get_attr('IFA_LOCAL')
-        elif msg['family'] == socket.AF_INET6:
+        elif msg['family'] == AF_INET6:
             addr = msg.get_attr('IFA_ADDRESS')
         else:
             return
@@ -1350,9 +1349,9 @@ class AddressesDict(dict):
             pass
 
     def _del(self, msg):
-        if msg['family'] == socket.AF_INET:
+        if msg['family'] == AF_INET:
             addr = msg.get_attr('IFA_LOCAL')
-        elif msg['family'] == socket.AF_INET6:
+        elif msg['family'] == AF_INET6:
             addr = msg.get_attr('IFA_ADDRESS')
         else:
             return
@@ -1375,7 +1374,7 @@ class NeighboursDict(dict):
             self._new(msg)
 
     def _new(self, msg):
-        if msg['family'] == socket.AF_BRIDGE:
+        if msg['family'] == AF_BRIDGE:
             return
 
         try:
@@ -1386,7 +1385,7 @@ class NeighboursDict(dict):
             pass
 
     def _del(self, msg):
-        if msg['family'] == socket.AF_BRIDGE:
+        if msg['family'] == AF_BRIDGE:
             return
         try:
             (self[msg['ifindex']]
