@@ -2,6 +2,7 @@
 '''
 import struct
 import threading
+from collections import OrderedDict
 from socket import inet_pton
 from socket import AF_INET
 from socket import AF_INET6
@@ -35,7 +36,7 @@ class LinkedSet(set):
         self.target = threading.Event()
         self.targets = {self.target: _check_default_target}
         self._ct = None
-        self.raw = {}
+        self.raw = OrderedDict()
         self.links = []
         self.exclusive = set()
 
@@ -51,7 +52,7 @@ class LinkedSet(set):
                 target.clear()
                 del self.targets[target]
 
-    def set_target(self, value):
+    def set_target(self, value, ignore_state=False):
         '''
         Set target state for the object and clear the target
         event. Once the target is reached, the event will be
@@ -67,11 +68,13 @@ class LinkedSet(set):
                 # immediately check, if the target already
                 # reached -- otherwise you will miss the
                 # target forever
-                self.check_target()
+                if not ignore_state:
+                    self.check_target()
             elif hasattr(value, '__call__'):
                 new_target = threading.Event()
                 self.targets[new_target] = value
-                self.check_target()
+                if not ignore_state:
+                    self.check_target()
                 return new_target
             else:
                 raise TypeError("target type not supported")
@@ -150,7 +153,7 @@ class LinkedSet(set):
         self.links.remove(link)
 
     def __repr__(self):
-        return repr(list(self))
+        return repr(tuple(self))
 
 
 class IPaddrSet(LinkedSet):
@@ -158,16 +161,36 @@ class IPaddrSet(LinkedSet):
     LinkedSet child class with different target filter. The
     filter ignores link local IPv6 addresses when sets and checks
     the target.
+
+    The `wait_ip()` routine by default does not ignore link local
+    IPv6 addresses, but it may be changed with the `ignore_link_local`
+    argument.
     '''
-    def wait_ip(self, net, mask=None, timeout=None):
+    @property
+    def ipv4(self):
+        ret = IPaddrSet()
+        for x in self:
+            if self[x]['family'] == AF_INET:
+                ret.add(x, self[x])
+        return ret
+
+    @property
+    def ipv6(self):
+        ret = IPaddrSet()
+        for x in self:
+            if self[x]['family'] == AF_INET6:
+                ret.add(x, self[x])
+        return ret
+
+    def wait_ip(self, net, mask=None, timeout=None, ignore_link_local=False):
         family = AF_INET6 if net.find(':') >= 0 else AF_INET
         alen = 32 if family == AF_INET else 128
         net = inet_pton(family, net)
+        if mask is None:
+            mask = alen
         if family == AF_INET:
-            mask = mask or 32
             net = struct.unpack('>I', net)[0]
         else:
-            mask = mask or 128
             na, nb = struct.unpack('>QQ', net)
             net = (na << 64) | nb
         match = net & (((1 << mask) - 1) << (alen - mask))
@@ -177,12 +200,17 @@ class IPaddrSet(LinkedSet):
                 rfamily = AF_INET6 if rnet.find(':') >= 0 else AF_INET
                 if family != rfamily:
                     continue
+                if family == AF_INET6 and \
+                        ignore_link_local and \
+                        rnet[:4] == 'fe80' and \
+                        rmask == 64:
+                    continue
                 rnet = inet_pton(family, rnet)
                 if family == AF_INET:
                     rnet = struct.unpack('>I', rnet)[0]
                 else:
                     rna, rnb = struct.unpack('>QQ', rnet)
-                    rnet = (na << 64) | nb
+                    rnet = (rna << 64) | rnb
                 if (rnet & (((1 << mask) - 1) << (alen - mask))) == match:
                     return True
             return False
@@ -192,12 +220,6 @@ class IPaddrSet(LinkedSet):
         self.clear_target(target)
         return ret
 
-    def target_filter(self, x):
-        return not ((x[0][:4] == 'fe80') and (x[1] == 64))
-
-    def __repr__(self):
-        return repr(['%s/%s' % x for x in self])
-
     def __getitem__(self, key):
         if isinstance(key, (tuple, list)):
             return self.raw[key]
@@ -209,3 +231,70 @@ class IPaddrSet(LinkedSet):
             return self.raw[key]
         else:
             TypeError('wrong key type')
+
+
+class SortedIPaddrSet(IPaddrSet):
+    def __init__(self, *argv, **kwarg):
+        super(SortedIPaddrSet, self).__init__(*argv, **kwarg)
+        if argv and isinstance(argv[0], SortedIPaddrSet):
+            # Re-initialize self.raw from argv[0].raw to preserve order:
+            self.raw = OrderedDict(argv[0].raw)
+
+    def __and__(self, other):
+        nset = SortedIPaddrSet(self)
+        return nset.__iand__(other)
+
+    def __iand__(self, other):
+        for key in self.raw:
+            if key not in other:
+                self.remove(key)
+        return self
+
+    def __rand__(self, other):
+        return self.__and__(other)
+
+    def __xor__(self, other):
+        nset = SortedIPaddrSet(self)
+        return nset.__ixor__(other)
+
+    def __ixor__(self, other):
+        if not isinstance(other, SortedIPaddrSet):
+            return RuntimeError('SortedIPaddrSet instance required')
+        xor_keys = set(self.raw.keys()) ^ set(other.raw.keys())
+        for key in xor_keys:
+            if key in self:
+                self.remove(key)
+            else:
+                self.add(key, raw=other.raw[key], cascade=False)
+        return self
+
+    def __rxor__(self, other):
+        return self.__xor__(other)
+
+    def __or__(self, other):
+        nset = SortedIPaddrSet(self)
+        return nset.__ior__(other)
+
+    def __ior__(self, other):
+        if not isinstance(other, SortedIPaddrSet):
+            return RuntimeError('SortedIPaddrSet instance required')
+        for key, value in other.raw.items():
+            if key not in self:
+                self.add(key, raw=value, cascade=False)
+        return self
+
+    def __ror__(self, other):
+        return self.__or__(other)
+
+    def __sub__(self, other):
+        nset = SortedIPaddrSet(self)
+        return nset.__isub__(other)
+
+    def __isub__(self, other):
+        for key in other:
+            if key in self:
+                self.remove(key)
+        return self
+
+    def __iter__(self):
+        return iter(self.raw)
