@@ -391,6 +391,19 @@ Please notice, that the device can be specified with `oif`
 (output interface) or `iif` (input interface), the `device`
 keyword is not supported anymore.
 
+More examples::
+
+    # specify table and priority
+    (ipdb.routes
+     .add(dst='172.16.1.0/24',
+          gateway='192.168.0.1',
+          table=100,
+          priority=10)
+     .commit())
+
+The `priority` field is what the `iproute2` utility calls
+`metric` -- see also below.
+
 Get a route
 ~~~~~~~~~~~
 
@@ -428,9 +441,11 @@ of 'prefix/mask' string and the route priority (if any)::
 
     In [1]: ipdb.routes.tables[254].idx.keys()
     Out[1]:
-    [RouteKey(dst='default', priority=600),
-     RouteKey(dst='10.1.0.0/24', priority=600),
-     RouteKey(dst='192.168.122.0/24', priority=None)]
+    [RouteKey(dst='default', scope=0, table=254, family=2, ...),
+     RouteKey(dst='172.17.0.0/16', scope=253, table=254, ...),
+     RouteKey(dst='172.16.254.0/24', scope=253, table=254, ...),
+     RouteKey(dst='192.168.122.0/24', scope=253, table=254, ...),
+     RouteKey(dst='fe80::/64', scope=0, table=254, family=10, ...)]
 
 But a routing table in IPDB allows several variants of the
 route spec. The simplest case is to retrieve a route by
@@ -454,8 +469,23 @@ records and filter by a key to retrieve all matches::
 
 It is also possible to use dicts as specs::
 
-    ipdb.routes[{'dst': '172.16.0.0/16',
-                 'oif': 2}]
+    # get IPv4 default route
+    ipdb.routes[{'dst': 'default', 'family': AF_INET}]
+
+    # get IPv6 default route
+    ipdb.routes[{'dst': 'default', 'family': AF_INET6}]
+
+    # get route by priority
+    ipdb.routes.table[100][{'dst': '10.0.0.0/24', 'priority': 10}]
+
+While this notation returns one route, there is a method to get
+all the routes matching the spec::
+
+    # get all the routes from all the tables via some interface
+    ipdb.routes.filter({'oif': idx})
+
+    # get all IPv6 routes from some table
+    ipdb.routes.table[tnum].filter({'family': AF_INET6})
 
 Route metrics
 ~~~~~~~~~~~~~
@@ -945,6 +975,8 @@ class IPDB(object):
             self._post_callbacks[safe.uuid] = safe
         elif mode == 'pre':
             self._pre_callbacks[safe.uuid] = safe
+        else:
+            raise KeyError('Unknown callback mode')
         return safe.uuid
 
     def unregister_callback(self, cuid, mode='post'):
@@ -987,7 +1019,7 @@ class IPDB(object):
                         msg.reset()
                         self.mnl.put(msg, RTM_GETLINK)
                     except Exception as e:
-                        logging.warning("shutdown error: %s", e)
+                        log.warning("shutdown error: %s", e)
                         # Just give up.
                         # We can not handle this case
 
@@ -1065,39 +1097,75 @@ class IPDB(object):
         else:
             raise NotImplementedError()
 
-    def review(self):
-        ilist = {}
-        rlist = {}
-        ret = {}
+    def items(self):
+        # TODO: add support for filters?
 
-        # collect interface transactions
+        # iterate interfaces
         for ifname in getattr(self, 'by_name', {}):
-            try:
-                ilist[ifname] = self.interfaces[ifname].review()
-            except TypeError:
-                pass
-        # collect route transactions
+            yield (('interfaces', ifname), self.interfaces[ifname])
+
+        # iterate routes
         for table in getattr(getattr(self, 'routes', None),
                              'tables', {}):
-            for key in self.routes.tables[table].keys():
-                try:
-                    rlist[ifname] = self.routes.tables[table][key].review()
-                except TypeError:
-                    pass
+            for key, route in self.routes.tables[table].items():
+                yield (('routes', table, key), route)
 
-        if ilist:
-            ret['interfaces'] = ilist
-        if rlist:
-            ret['routes'] = rlist
+    def dump(self):
+        ret = {}
+        for key, obj in self.items():
+            ptr = ret
+            for step in key[:-1]:
+                if step not in ptr:
+                    ptr[step] = {}
+                ptr = ptr[step]
+            ptr[key[-1]] = obj
+        return ret
+
+    def load(self, config, ptr=None):
+        if ptr is None:
+            ptr = self
+
+        for key in config:
+            obj = getattr(ptr, key, None)
+            if obj is not None:
+                if hasattr(obj, 'load'):
+                    obj.load(config[key])
+                else:
+                    self.load(config[key], ptr=obj)
+            elif hasattr(ptr, 'add'):
+                ptr.add(**config[key])
+
+        return self
+
+    def review(self):
+        ret = {}
+        for key, obj in self.items():
+            ptr = ret
+            try:
+                rev = obj.review()
+            except TypeError:
+                continue
+
+            for step in key[:-1]:
+                if step not in ptr:
+                    ptr[step] = {}
+                ptr = ptr[step]
+            ptr[key[-1]] = rev
 
         if not ret:
-            raise TypeError('no transactions started')
+            raise TypeError('no transaction started')
         return ret
 
     def drop(self):
-        for (system, tx) in self.review().items():
-            for key in tx:
-                getattr(self, system)[key].drop()
+        ok = False
+        for key, obj in self.items():
+            try:
+                obj.drop()
+            except TypeError:
+                continue
+            ok = True
+        if not ok:
+            raise TypeError('no transaction started')
 
     def commit(self, transactions=None, phase=1):
         # what to commit: either from transactions argument, or from
@@ -1198,6 +1266,8 @@ class IPDB(object):
                 for (target, tx) in transactions:
                     target.drop(tx.uid)
 
+        return self
+
     def watchdog(self, action='RTM_NEWLINK', **kwarg):
         return Watchdog(self, action, kwarg)
 
