@@ -8,6 +8,7 @@ from socket import AF_UNSPEC
 from socket import AF_INET6
 from socket import AF_INET
 from socket import inet_pton
+from socket import inet_ntop
 from pyroute2.common import AF_MPLS
 from pyroute2.common import basestring
 from pyroute2.netlink import rtnl
@@ -29,6 +30,7 @@ log = logging.getLogger(__name__)
 groups = rtnl.RTNLGRP_IPV4_ROUTE |\
     rtnl.RTNLGRP_IPV6_ROUTE |\
     rtnl.RTNLGRP_MPLS_ROUTE
+IP6_RT_PRIO_USER = 1024
 
 
 class Metrics(Transactional):
@@ -133,11 +135,12 @@ class WatchdogKey(dict):
 # Universal route key
 RouteKey = namedtuple('RouteKey',
                       ('dst',
-                       'scope',
                        'table',
+                       'family',
                        'priority',
+                       'tos',
                        'oif'))
-RouteKey._required = 3  # number of required fields (should go first)
+RouteKey._required = 5  # number of required fields (should go first)
 
 # IP multipath NH key
 IPNHKey = namedtuple('IPNHKey',
@@ -154,6 +157,27 @@ MPLSNHKey = namedtuple('MPLSNHKey',
 MPLSNHKey._required = 2
 
 
+def _normalize_ipaddr(x, y):
+    if isinstance(y, basestring) and y.find(':') > -1:
+        y = inet_ntop(AF_INET6, inet_pton(AF_INET6, y))
+    return x == y
+
+
+def _normalize_ipnet(x, y):
+    #
+    # x -- incoming value
+    # y -- transaction value
+    #
+    if isinstance(y, basestring) and y.find(':') > -1:
+        s = y.split('/')
+        ip = inet_ntop(AF_INET6, inet_pton(AF_INET6, s[0]))
+        if len(s) > 1:
+            y = '%s/%s' % (ip, s[1])
+        else:
+            y = ip
+    return x == y
+
+
 class BaseRoute(Transactional):
     '''
     Persistent transactional route object
@@ -172,6 +196,10 @@ class BaseRoute(Transactional):
                'header',
                'event',
                'cacheinfo')
+    _fields_cmp = {'src': _normalize_ipnet,
+                   'dst': _normalize_ipnet,
+                   'gateway': _normalize_ipaddr,
+                   'prefsrc': _normalize_ipaddr}
 
     def __init__(self, ipdb, mode=None, parent=None, uid=None):
         Transactional.__init__(self, ipdb, mode, parent, uid)
@@ -488,6 +516,13 @@ class BaseRoute(Transactional):
         self['ipdb_scope'] = 'shadow'
         return self
 
+    def detach(self):
+        if self.get('family') == AF_MPLS:
+            table = 'mpls'
+        else:
+            table = self.get('table', 254)
+        del self.ipdb.routes.tables[table][self.make_key(self)]
+
 
 class Route(BaseRoute):
     _nested = ['encap', 'metrics']
@@ -565,17 +600,35 @@ class Route(BaseRoute):
         if isinstance(msg, nlmsg_base):
             for field in RouteKey._fields:
                 v = msg.get_attr(msg.name2nla(field))
-                if field in ('src', 'dst'):
+                if field == 'dst':
                     if v is not None:
-                        v = '%s/%s' % (v, msg['%s_len' % field])
-                    elif field == 'dst':
+                        v = '%s/%s' % (v, msg['dst_len'])
+                    else:
                         v = 'default'
+                elif field == 'tos' and msg.get('family') != AF_INET:
+                    # ignore tos field for non-IPv4 routes,
+                    # as it used as a key only there
+                    v = None
                 elif v is None:
                     v = msg.get(field, None)
                 values.append(v)
         elif isinstance(msg, dict):
             for field in RouteKey._fields:
-                values.append(msg.get(field, None))
+                v = msg.get(field, None)
+                if field == 'dst' and \
+                        isinstance(v, basestring) and \
+                        v.find(':') > -1:
+                    v = v.split('/')
+                    ip = inet_ntop(AF_INET6, inet_pton(AF_INET6, v[0]))
+                    if len(v) > 1:
+                        v = '%s/%s' % (ip, v[1])
+                    else:
+                        v = ip
+                elif field == 'tos' and msg.get('family') != AF_INET:
+                    # ignore tos field for non-IPv4 routes,
+                    # as it used as a key only there
+                    v = None
+                values.append(v)
         else:
             raise TypeError('prime not supported: %s' % type(msg))
         return RouteKey(*values)
@@ -763,6 +816,10 @@ class RoutingTable(object):
         with self.lock:
             return [x['route'][key] for x in self.__nogc__()]
 
+    def items(self):
+        for key in self.keys():
+            yield (key, self[key])
+
     def filter(self, target, oneshot=False):
         #
         if isinstance(target, types.FunctionType):
@@ -946,12 +1003,25 @@ class RoutingTableSet(object):
         Create a route from a dictionary
         '''
         spec = dict(spec or kwarg)
+        if 'tos' not in spec:
+            spec['tos'] = 0
         if 'scope' not in spec:
             spec['scope'] = 0
         if 'table' not in spec:
             spec['table'] = 254
+        if 'family' not in spec:
+            if (spec.get('dst', '').find(':') > -1) or \
+                    (spec.get('gateway', '').find(':') > -1):
+                spec['family'] = AF_INET6
+            else:
+                spec['family'] = AF_INET
         if 'dst' not in spec:
             raise ValueError('dst not specified')
+        if 'priority' not in spec:
+            if spec['family'] == AF_INET6:
+                spec['priority'] = IP6_RT_PRIO_USER
+            else:
+                spec['priority'] = None
         multipath = spec.pop('multipath', [])
         if spec.get('family', 0) == AF_MPLS:
             table = 'mpls'
@@ -991,9 +1061,7 @@ class RoutingTableSet(object):
         if msg['family'] == AF_MPLS:
             table = 'mpls'
         else:
-            table = msg['table']
-            if table == 252:
-                table = msg.get_attr('RTA_TABLE')
+            table = msg.get_attr('RTA_TABLE', msg['table'])
 
         if table in self.ignore_rtables:
             return
