@@ -5,7 +5,8 @@ Generic netlink
 
 Describe
 '''
-
+import errno
+import logging
 from pyroute2.netlink import CTRL_CMD_GETFAMILY
 from pyroute2.netlink import GENL_ID_CTRL
 from pyroute2.netlink import NLM_F_REQUEST
@@ -65,9 +66,12 @@ class GenericNetlinkSocket(NetlinkSocket):
         msg['header']['flags'] = NLM_F_REQUEST
         msg['header']['pid'] = self.pid
         msg.encode()
-        self.sendto(msg.buf.getvalue(), (0, 0))
+        self.sendto(msg.data, (0, 0))
         msg = self.get()[0]
         err = msg['header'].get('error', None)
         if err is not None:
+            if hasattr(err, 'code') and err.code == errno.ENOENT:
+                logging.error('Generic netlink protocol %s not found' % proto)
+                logging.error('Please check if the protocol module is loaded')
             raise err
         return msg
