@@ -89,7 +89,6 @@ import logging
 import traceback
 import threading
 
-from socket import AF_NETLINK
 from socket import SOCK_DGRAM
 from socket import MSG_PEEK
 from socket import SOL_SOCKET
@@ -97,6 +96,7 @@ from socket import SO_RCVBUF
 from socket import SO_SNDBUF
 
 from pyroute2 import config
+from pyroute2.config import AF_NETLINK
 from pyroute2.common import AddrPool
 from pyroute2.common import DEFAULT_RCVBUF
 from pyroute2.netlink import nlmsg
