@@ -6,17 +6,40 @@ from socket import AF_INET
 from socket import AF_INET6
 from pyroute2.common import AF_MPLS
 from pyroute2.common import hexdump
+from pyroute2.common import map_namespace
 from pyroute2.netlink import nlmsg
 from pyroute2.netlink import nla
-from pyroute2.netlink import nlmsg_base
+
+RTNH_F_DEAD = 1
+RTNH_F_PERVASIVE = 2
+RTNH_F_ONLINK = 4
+RTNH_F_OFFLOAD = 8
+RTNH_F_LINKDOWN = 16
+(RTNH_F_NAMES, RTNH_F_VALUES) = map_namespace('RTNH_F', globals())
 
 
-class nh_header(nlmsg_base):
-    align = 2
-    fields = (('length', 'H'), )
+class nlflags(object):
+
+    def encode(self):
+        if isinstance(self['flags'], (set, tuple, list)):
+            self['flags'] = self.names2flags(self['flags'])
+        return super(nlflags, self).encode()
+
+    def flags2names(self, flags=None):
+        ret = []
+        for flag in RTNH_F_VALUES:
+            if (flag & flags) == flag:
+                ret.append(RTNH_F_VALUES[flag].lower()[7:])
+        return ret
+
+    def names2flags(self, flags=None):
+        ret = 0
+        for flag in flags or self['flags']:
+            ret |= RTNH_F_NAMES['RTNH_F_' + flag.upper()]
+        return ret
 
 
-class rtmsg(nlmsg):
+class rtmsg_base(nlflags):
     '''
     Route message
 
@@ -36,6 +59,9 @@ class rtmsg(nlmsg):
             unsigned int  rtm_flags;
         };
     '''
+
+    __slots__ = ()
+
     prefix = 'RTA_'
 
     fields = (('family', 'B'),
@@ -57,9 +83,9 @@ class rtmsg(nlmsg):
                ('RTA_PRIORITY', 'uint32'),
                ('RTA_PREFSRC', 'target'),
                ('RTA_METRICS', 'metrics'),
-               ('RTA_MULTIPATH', '*nh'),
+               ('RTA_MULTIPATH', '*get_nh'),
                ('RTA_PROTOINFO', 'uint32'),
-               ('RTA_FLOW', 'hex'),
+               ('RTA_FLOW', 'uint32'),
                ('RTA_CACHEINFO', 'cacheinfo'),
                ('RTA_SESSION', 'hex'),
                ('RTA_MP_ALGO', 'hex'),
@@ -68,14 +94,34 @@ class rtmsg(nlmsg):
                ('RTA_MFC_STATS', 'rta_mfc_stats'),
                ('RTA_VIA', 'rtvia'),
                ('RTA_NEWDST', 'target'),
-               ('RTA_PREF', 'hex'))
+               ('RTA_PREF', 'hex'),
+               ('RTA_ENCAP_TYPE', 'uint16'),
+               ('RTA_ENCAP', 'encap_info'),
+               ('RTA_EXPIRES', 'hex'))
+
+    @staticmethod
+    def encap_info(self, *argv, **kwarg):
+        return self.mpls_encap_info
+
+    class mpls_encap_info(nla):
+
+        __slots__ = ()
+
+        nla_map = (('MPLS_IPTUNNEL_UNSPEC', 'none'),
+                   ('MPLS_IPTUNNEL_DST', 'mpls_target'))
 
     class rta_mfc_stats(nla):
+
+        __slots__ = ()
+
         fields = (('mfcs_packets', 'uint64'),
                   ('mfcs_bytes', 'uint64'),
                   ('mfcs_wrong_if', 'uint64'))
 
     class metrics(nla):
+
+        __slots__ = ()
+
         prefix = 'RTAX_'
         nla_map = (('RTAX_UNSPEC', 'none'),
                    ('RTAX_LOCK', 'uint32'),
@@ -94,23 +140,23 @@ class rtmsg(nlmsg):
                    ('RTAX_INITRWND', 'uint32'),
                    ('RTAX_QUICKACK', 'uint32'))
 
-    class nh(nla):
-        align = 2
-        cell_header = nh_header
-        fields = (('flags', 'B'),
-                  ('hops', 'B'),
-                  ('ifindex', 'i'))
-        nla_map = ((5, 'RTA_GATEWAY', 'target'),
-                   (11, 'RTA_FLOW', 'hex'),
-                   (18, 'RTA_VIA', 'hex'))
+    @staticmethod
+    def get_nh(self, *argv, **kwarg):
+        return nh
 
     class rtvia(nla):
+
+        __slots__ = ()
+
         fields = (('value', 's'), )
 
         def encode(self):
             family = self.get('family', AF_UNSPEC)
             if family in (AF_INET, AF_INET6):
                 addr = inet_pton(family, self['addr'])
+            else:
+                raise TypeError('Family %s not supported for RTA_VIA'
+                                % family)
             self['value'] = struct.pack('H', family) + addr
             nla.encode(self)
 
@@ -127,6 +173,9 @@ class rtmsg(nlmsg):
             self.value = {'family': family, 'addr': addr}
 
     class cacheinfo(nla):
+
+        __slots__ = ()
+
         fields = (('rta_clntref', 'I'),
                   ('rta_lastuse', 'I'),
                   ('rta_expires', 'i'),
@@ -136,6 +185,11 @@ class rtmsg(nlmsg):
                   ('rta_ts', 'I'),
                   ('rta_tsage', 'I'))
 
+
+class rtmsg(rtmsg_base, nlmsg):
+
+    __slots__ = ()
+
     def encode(self):
         if self.get('family') == AF_MPLS:
             # force fields
@@ -144,6 +198,21 @@ class rtmsg(nlmsg):
             self['type'] = 1
             # assert NLA types
             for n in self.get('attrs', []):
-                if n[0] not in ('RTA_OIF', 'RTA_DST', 'RTA_VIA', 'RTA_NEWDST'):
+                if n[0] not in ('RTA_OIF',
+                                'RTA_DST',
+                                'RTA_VIA',
+                                'RTA_NEWDST',
+                                'RTA_MULTIPATH'):
                     raise TypeError('Incorrect NLA type %s for AF_MPLS' % n[0])
-        nlmsg.encode(self)
+        super(rtmsg_base, self).encode()
+
+
+class nh(rtmsg_base, nla):
+
+    __slots__ = ()
+
+    is_nla = False
+    cell_header = (('length', 'H'), )
+    fields = (('flags', 'B'),
+              ('hops', 'B'),
+              ('oif', 'i'))
